package com.its.pis.dao.mapper;

import com.its.pis.vo.voStatisticsTime;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

@Mapper
public interface VdsStatMapper {

    int CRT_TB_VDS_DTCT_HS(@Param("prcs") voStatisticsTime paramStat);
    int CRT_TB_VDS_DTCT_15M_STAT(@Param("prcs") voStatisticsTime paramStat);
    int CRT_TB_VDS_DTCT_HH_STAT(@Param("prcs") voStatisticsTime paramStat);
    int CRT_TB_VDS_DTCT_DD_STAT(@Param("prcs") voStatisticsTime paramStat);
    int CRT_TB_VDS_DTCT_MN_STAT(@Param("prcs") voStatisticsTime paramStat);
    int CRT_TB_VDS_DTCT_YY_STAT(@Param("prcs") voStatisticsTime paramStat);
}
